/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.gui;

import java.io.IOException;

import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.util.ResourceLocation;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.recipes.anvil.AnvilRecipe;
import net.dries007.tfc.client.button.GuiButtonAnvilPlan;
import net.dries007.tfc.client.button.GuiButtonAnvilStep;
import net.dries007.tfc.client.button.IButtonTooltip;
import net.dries007.tfc.network.PacketGuiButton;
import net.dries007.tfc.objects.te.TEAnvilTFC;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.forge.ForgeRule;
import net.dries007.tfc.util.forge.ForgeStep;
import net.dries007.tfc.util.forge.ForgeSteps;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

public class GuiAnvilTFC extends GuiContainerTE<TEAnvilTFC>
{
    public static final ResourceLocation ANVIL_BACKGROUND = new ResourceLocation(MOD_ID, "textures/gui/anvil.png");
    public static final int BUTTON_ID_STEP_MIN = 0;
    public static final int BUTTON_ID_STEP_MAX = 7;
    public static final int BUTTON_ID_PLAN = 8;

    public GuiAnvilTFC(Container container, InventoryPlayer playerInv, TEAnvilTFC tile)
    {
        super(container, playerInv, tile, ANVIL_BACKGROUND);
        field_147000_g = 192;
    }

    @Override
    public void func_73866_w_()
    {
        super.func_73866_w_();

        int buttonID = -1;
        for (ForgeStep step : ForgeStep.values())
        {
            func_189646_b(new GuiButtonAnvilStep(++buttonID, field_147003_i, field_147009_r, step));
        }
        func_189646_b(new GuiButtonAnvilPlan(tile, ++buttonID, field_147003_i, field_147009_r));
    }

    @Override
    protected void func_191948_b(int mouseX, int mouseY)
    {
        // Button Tooltips
        for (GuiButton button : field_146292_n)
        {
            if (button instanceof IButtonTooltip && button.func_146115_a())
            {
                IButtonTooltip tooltip = (IButtonTooltip) button;
                if (tooltip.hasTooltip())
                {
                    func_146279_a(I18n.func_135052_a(tooltip.getTooltip()), mouseX, mouseY);
                }
            }
        }
        // Rule Tooltips
        AnvilRecipe recipe = tile.getRecipe();
        if (recipe != null)
        {
            int x = field_147003_i + 61, y = field_147009_r + 7;
            for (int i = 0; i < recipe.getRules().length; i++)
            {
                ForgeRule rule = recipe.getRules()[i];
                if (rule != null && mouseX >= x && mouseY >= y && mouseX < x + 18 && mouseY < y + 22)
                {
                    // Hovering over rule area
                    func_146279_a(Helpers.getEnumName(rule), mouseX, mouseY);
                    break;
                }
                x += 19;
            }
        }
        super.func_191948_b(mouseX, mouseY);
    }

    @Override
    protected void func_146976_a(float partialTicks, int mouseX, int mouseY)
    {
        super.func_146976_a(partialTicks, mouseX, mouseY);

        // Draw the progress indicators
        int progress = tile.getWorkingProgress();
        func_73729_b(field_147003_i + 13 + progress, field_147009_r + 100, 176, 0, 5, 5);

        int target = tile.getWorkingTarget();
        func_73729_b(field_147003_i + 13 + target, field_147009_r + 94, 181, 0, 5, 5);

        // Draw rule icons
        AnvilRecipe recipe = tile.getRecipe();
        ForgeSteps steps = tile.getSteps();
        if (recipe != null)
        {
            for (int i = 0; i < recipe.getRules().length; i++)
            {
                ForgeRule rule = recipe.getRules()[i];
                if (rule != null)
                {
                    int xOffset = i * 19;
                    // The rule icon
                    func_152125_a(field_147003_i + 64 + xOffset, field_147009_r + 10, rule.getU(), rule.getV(), 32, 32, 10, 10, 256, 256);

                    // The overlay
                    if (rule.matches(steps))
                    {
                        // GREEN
                        GlStateManager.func_179124_c(0f, 0.6f, 0.2f);
                    }
                    else
                    {
                        // RED
                        GlStateManager.func_179124_c(1f, 0.4f, 0);
                    }
                    func_73729_b(field_147003_i + 59 + xOffset, field_147009_r + 7, 198, rule.getW(), 20, 22);
                    GlStateManager.func_179124_c(1f, 1f, 1f);
                }
            }
        }

        // Draw step icons
        for (int i = 0; i < 3; i++)
        {
            ForgeStep step = steps.getStep(i);
            if (step != null)
            {
                // Reverses the placement of the steps to line up better with the rules
                int xOffset = (2 - i) * 19;
                func_152125_a(field_147003_i + 64 + xOffset, field_147009_r + 31, step.getU(), step.getV(), 32, 32, 10, 10, 256, 256);
            }
        }

    }

    @Override
    protected void func_146284_a(GuiButton button) throws IOException
    {
        TerraFirmaCraft.getNetwork().sendToServer(new PacketGuiButton(button.field_146127_k));
        super.func_146284_a(button);
    }

}
