/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.gui;

import java.io.IOException;

import org.lwjgl.opengl.GL11;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.Slot;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.client.button.GuiButtonLargeVesselSeal;
import net.dries007.tfc.client.button.IButtonTooltip;
import net.dries007.tfc.network.PacketGuiButton;
import net.dries007.tfc.objects.te.TELargeVessel;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

public class GuiLargeVessel extends GuiContainerTE<TELargeVessel>
{
    public static final ResourceLocation LARGE_VESSEL_BACKGROUND = new ResourceLocation(MOD_ID, "textures/gui/large_vessel.png");
    private final String translationKey;

    public GuiLargeVessel(Container container, InventoryPlayer playerInv, TELargeVessel tile, String translationKey)
    {
        super(container, playerInv, tile, LARGE_VESSEL_BACKGROUND);

        this.translationKey = translationKey;
    }

    @Override
    public void func_73866_w_()
    {
        super.func_73866_w_();
        func_189646_b(new GuiButtonLargeVesselSeal(tile, 0, field_147009_r, field_147003_i));
    }

    @Override
    protected void func_191948_b(int mouseX, int mouseY)
    {
        super.func_191948_b(mouseX, mouseY);

        // Button Tooltips
        for (GuiButton button : field_146292_n)
        {
            if (button instanceof IButtonTooltip && button.func_146115_a())
            {
                IButtonTooltip tooltip = (IButtonTooltip) button;
                if (tooltip.hasTooltip())
                {
                    func_146279_a(I18n.func_135052_a(tooltip.getTooltip()), mouseX, mouseY);
                }
            }
        }
    }

    @Override
    protected void func_146979_b(int mouseX, int mouseY)
    {
        String name = I18n.func_135052_a(translationKey + ".name");
        field_146289_q.func_78276_b(name, field_146999_f / 2 - field_146289_q.func_78256_a(name) / 2, 6, 0x404040);

        if (tile.isSealed())
        {
            // Draw over the input items, making them look unavailable
            IItemHandler handler = tile.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);
            if (handler != null)
            {
                GL11.glDisable(GL11.GL_DEPTH_TEST);
                for (int slotId = 0; slotId < handler.getSlots(); slotId++)
                {
                    drawSlotOverlay(field_147002_h.func_75139_a(slotId));
                }
                GL11.glEnable(GL11.GL_DEPTH_TEST);
            }

            // Draw the text displaying both the seal date, and the recipe name
            field_146289_q.func_78276_b(tile.getSealedDate(), field_146999_f / 2 - field_146289_q.func_78256_a(tile.getSealedDate()) / 2, 74, 0x404040);
        }
    }

    @Override
    protected void func_146976_a(float partialTicks, int mouseX, int mouseY)
    {
        super.func_146976_a(partialTicks, mouseX, mouseY);
    }

    @Override
    protected void func_146284_a(GuiButton button) throws IOException
    {
        if (button instanceof GuiButtonLargeVesselSeal)
        {
            TerraFirmaCraft.getNetwork().sendToServer(new PacketGuiButton(button.field_146127_k));
        }
        super.func_146284_a(button);
    }

    private void drawSlotOverlay(Slot slot)
    {
        int xPos = slot.field_75223_e - 1;
        int yPos = slot.field_75221_f - 1;

        this.func_73733_a(xPos, yPos, xPos + 18, yPos + 18, 0x75FFFFFF, 0x75FFFFFF);
    }
}
