/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.model.animal;

import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.client.model.ModelBase;
import net.minecraft.client.model.ModelRenderer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.entity.Entity;
import net.minecraft.util.math.MathHelper;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

@SideOnly(Side.CLIENT)
@ParametersAreNonnullByDefault
public class ModelBearTFC extends ModelBase
{
    private ModelRenderer neck;
    private ModelRenderer bearHead;
    private ModelRenderer body;
    private ModelRenderer mainBody;
    private ModelRenderer leg1;
    private ModelRenderer leg2;
    private ModelRenderer leg3;
    private ModelRenderer leg4;
    private ModelRenderer tail;
    private ModelRenderer ear1;
    private ModelRenderer ear2;
    private ModelRenderer nose;

    public ModelBearTFC()
    {
        field_78090_t = 128;
        field_78089_u = 64;

        neck = new ModelRenderer(this, 0, 29);
        neck.func_78789_a(0F, 0F, 0F, 4, 5, 8);
        neck.func_78793_a(-3F, 12F, -10F);
        neck.func_78787_b(128, 64);
        neck.field_78809_i = true;
        setRotation(neck, 0.3839724F, 0F, 0F);
        bearHead = new ModelRenderer(this, 0, 0);
        bearHead.func_78789_a(-3F, -3F, -2F, 6, 6, 4);
        bearHead.func_78793_a(-1F, 13.5F, -10F);
        bearHead.func_78787_b(128, 64);
        bearHead.field_78809_i = true;
        setRotation(bearHead, 0F, 0F, 0F);
        body = new ModelRenderer(this, 24, 18);
        body.func_78789_a(-4F, -2F, -3F, 6, 7, 7);
        body.func_78793_a(0F, 14F, 4F);
        body.func_78787_b(128, 64);
        body.field_78809_i = true;
        setRotation(body, 1.308997F, 0F, 0F);
        mainBody = new ModelRenderer(this, 24, 0);
        mainBody.func_78789_a(-4F, -3F, -3F, 8, 10, 8);
        mainBody.func_78793_a(-1F, 14F, -3F);
        mainBody.func_78787_b(128, 64);
        mainBody.field_78809_i = true;
        setRotation(mainBody, 1.570796F, 0F, 0F);
        leg1 = new ModelRenderer(this, 0, 18);
        leg1.func_78789_a(-1F, 0F, -1F, 2, 8, 3);
        leg1.func_78793_a(-2.966667F, 16F, 6F);
        leg1.func_78787_b(128, 64);
        leg1.field_78809_i = true;
        setRotation(leg1, 0F, 0F, 0F);
        leg2 = new ModelRenderer(this, 0, 18);
        leg2.func_78789_a(-0.4666667F, 0F, -1F, 2, 8, 3);
        leg2.func_78793_a(0.5F, 16F, 6F);
        leg2.func_78787_b(128, 64);
        leg2.field_78809_i = true;
        setRotation(leg2, 0F, 0F, 0F);
        leg3 = new ModelRenderer(this, 0, 18);
        leg3.func_78789_a(-1F, 0F, -1F, 2, 8, 3);
        leg3.func_78793_a(-3.5F, 16F, -4F);
        leg3.func_78787_b(128, 64);
        leg3.field_78809_i = true;
        setRotation(leg3, 0F, 0F, 0F);
        leg4 = new ModelRenderer(this, 0, 18);
        leg4.func_78789_a(-1F, 0F, -1F, 2, 8, 3);
        leg4.func_78793_a(1.5F, 16F, -4F);
        leg4.func_78787_b(128, 64);
        leg4.field_78809_i = true;
        setRotation(leg4, 0F, 0F, 0F);
        tail = new ModelRenderer(this, 10, 18);
        tail.func_78789_a(-1F, 0F, -1F, 2, 2, 2);
        tail.func_78793_a(-1F, 13F, 9F);
        tail.func_78787_b(128, 64);
        tail.field_78809_i = true;
        setRotation(tail, 1.130069F, 0F, 0F);
        ear1 = new ModelRenderer(this, 16, 14);
        ear1.func_78789_a(-3F, -5F, 0F, 2, 2, 1);
        ear1.func_78793_a(-1F, 13.5F, -10F);
        ear1.func_78787_b(128, 64);
        ear1.field_78809_i = true;
        setRotation(ear1, 0F, 0F, 0F);
        ear2 = new ModelRenderer(this, 16, 14);
        ear2.func_78789_a(1F, -5F, 0F, 2, 2, 1);
        ear2.func_78793_a(-1F, 13.5F, -10F);
        ear2.func_78787_b(128, 64);
        ear2.field_78809_i = true;
        setRotation(ear2, 0F, 0F, 0F);
        nose = new ModelRenderer(this, 0, 10);
        nose.func_78789_a(-1.5F, 0F, -4F, 3, 3, 4);
        nose.func_78793_a(-1.0F, 13.5F, -10F);
        nose.func_78787_b(128, 64);
        nose.field_78809_i = true;
        setRotation(nose, 0F, 0F, 0F);
    }


    @Override
    public void func_78088_a(Entity entity, float f, float f1, float f2, float f3, float f4, float f5)
    {
        super.func_78088_a(entity, f, f1, f2, f3, f4, f5);
        func_78087_a(f, f1, f2, f3, f4, f5, entity);

        float age = 1;

        if (field_78091_s)
        {
            float aa = 2F - (1.0F - age);
            GlStateManager.func_179094_E();
            float ab = (float) Math.sqrt(1.0F / aa);
            GlStateManager.func_179152_a(ab, ab, ab);
            GlStateManager.func_179109_b(0.0F, 24F * f5 * age / aa, 2F * f5 * age / ab);
            ear1.func_78785_a(f5);
            ear2.func_78785_a(f5);
            nose.func_78785_a(f5);
            bearHead.func_78785_a(f5);
            GlStateManager.func_179121_F();
            GlStateManager.func_179094_E();
            GlStateManager.func_179152_a(1.0F / aa, 1.0F / aa, 1.0F / aa);
            GlStateManager.func_179109_b(0.0F, 24F * f5 * age, 0.0F);
            body.func_78785_a(f5);
            mainBody.func_78785_a(f5);
            leg1.func_78785_a(f5);
            leg2.func_78785_a(f5);
            leg3.func_78785_a(f5);
            leg4.func_78785_a(f5);
            neck.func_78785_a(f5);
            tail.func_78785_a(f5);
            GlStateManager.func_179121_F();
        }
        else
        {
            ear1.func_78785_a(f5);
            ear2.func_78785_a(f5);
            nose.func_78785_a(f5);
            bearHead.func_78785_a(f5);
            body.func_78785_a(f5);
            mainBody.func_78785_a(f5);
            leg1.func_78785_a(f5);
            leg2.func_78785_a(f5);
            leg3.func_78785_a(f5);
            leg4.func_78785_a(f5);
            neck.func_78785_a(f5);
            tail.func_78785_a(f5);
        }
    }

    @Override
    public void func_78087_a(float f, float f1, float f2, float f3, float f4, float f5, Entity entity)
    {
        super.func_78087_a(f, f1, f2, f3, f4, f5, entity);
        bearHead.field_78795_f = f4 / (180F / (float) Math.PI);
        bearHead.field_78796_g = f3 / (180F / (float) Math.PI);

        ear1.field_78795_f = f4 / (180F / (float) Math.PI);
        ear1.field_78796_g = f3 / (180F / (float) Math.PI);

        ear2.field_78795_f = f4 / (180F / (float) Math.PI);
        ear2.field_78796_g = f3 / (180F / (float) Math.PI);

        nose.field_78795_f = f4 / (180F / (float) Math.PI);
        nose.field_78796_g = f3 / (180F / (float) Math.PI);

        leg1.field_78795_f = MathHelper.func_76134_b(f * 0.6662F) * 1.4F * f1;
        leg2.field_78795_f = MathHelper.func_76134_b(f * 0.6662F + (float) Math.PI) * 1.4F * f1;
        leg3.field_78795_f = MathHelper.func_76134_b(f * 0.6662F + (float) Math.PI) * 1.4F * f1;
        leg4.field_78795_f = MathHelper.func_76134_b(f * 0.6662F) * 1.4F * f1;
    }

    private void setRotation(ModelRenderer model, float x, float y, float z)
    {
        model.field_78795_f = x;
        model.field_78796_g = y;
        model.field_78808_h = z;
    }
}
