/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.model.animal;

import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.client.model.ModelBase;
import net.minecraft.client.model.ModelRenderer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.entity.Entity;
import net.minecraft.util.math.MathHelper;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.objects.entity.animal.EntityPheasantTFC;

@SideOnly(Side.CLIENT)
@ParametersAreNonnullByDefault
public class ModelPheasantTFC extends ModelBase
{
    private ModelRenderer body;
    private ModelRenderer tail;
    private ModelRenderer leftLeg;
    private ModelRenderer rightLeg;
    private ModelRenderer tailFeather;
    private ModelRenderer leftLowerLeg;
    private ModelRenderer rightLowerLeg;
    private ModelRenderer neck;
    private ModelRenderer head;
    private ModelRenderer beak;
    private ModelRenderer chest;
    private ModelRenderer leftWing;
    private ModelRenderer rightWing;
    private ModelRenderer leftFoot;
    private ModelRenderer rightFoot;

    public ModelPheasantTFC()
    {
        field_78090_t = 64;
        field_78089_u = 32;

        body = new ModelRenderer(this, 0, 7);
        body.func_78789_a(-4F, 0F, -2.5F, 9, 6, 5);
        body.func_78793_a(0F, 12F, 0F);
        body.field_78809_i = true;

        tail = new ModelRenderer(this, 0, 0);
        tail.func_78789_a(-7F, 2F, -2F, 6, 3, 4);
        tail.func_78793_a(0F, 12F, 0F);
        tail.field_78809_i = true;

        leftLeg = new ModelRenderer(this, 16, 24);
        leftLeg.func_78789_a(1F, 4F, 1F, 3, 4, 2);
        leftLeg.func_78793_a(0F, 12F, 0F);
        leftLeg.field_78809_i = true;

        rightLeg = new ModelRenderer(this, 26, 24);
        rightLeg.func_78789_a(1F, 4F, -3F, 3, 4, 2);
        rightLeg.func_78793_a(0F, 12F, 0F);
        rightLeg.field_78809_i = true;

        tailFeather = new ModelRenderer(this, 20, 0);
        tailFeather.func_78789_a(-14.96F, 3F, -1F, 9, 1, 2);
        tailFeather.func_78793_a(0F, 12F, 0F);
        tailFeather.field_78809_i = true;

        leftLowerLeg = new ModelRenderer(this, 34, 19);
        leftLowerLeg.func_78789_a(-1F, 8F, 1.5F, 1, 4, 1);
        leftLowerLeg.func_78793_a(0F, 0F, 0F);
        leftLowerLeg.field_78809_i = true;

        rightLowerLeg = new ModelRenderer(this, 38, 19);
        rightLowerLeg.func_78789_a(-1F, 8F, -2.5F, 1, 4, 1);
        rightLowerLeg.func_78793_a(0F, 0F, 0F);
        rightLowerLeg.field_78809_i = true;

        neck = new ModelRenderer(this, 28, 13);
        neck.func_78790_a(4F, 2F, -1.5F, 4, 3, 3, 0.05F);
        neck.func_78793_a(-4F, 2F, 0F);
        neck.field_78809_i = true;

        head = new ModelRenderer(this, 16, 18);
        head.func_78790_a(2.5F, -3F, -1.5F, 3, 3, 3, 0.1F);
        head.func_78793_a(4F, 10F, 0F);
        head.field_78809_i = true;

        beak = new ModelRenderer(this, 28, 19);
        beak.func_78789_a(5F, -2F, -0.5F, 2, 1, 1);
        beak.func_78793_a(4F, 10F, 0F);
        beak.field_78809_i = true;

        chest = new ModelRenderer(this, 28, 7);
        chest.func_78789_a(-1F, 4.5F, -1.5F, 4, 3, 3);
        chest.func_78793_a(-4F, 2F, 0F);
        chest.field_78809_i = true;

        leftWing = new ModelRenderer(this, 0, 23);
        leftWing.func_78789_a(-3F, 1F, 2.5F, 7, 4, 1);
        leftWing.func_78793_a(0F, 12F, 0F);
        leftWing.field_78809_i = true;

        rightWing = new ModelRenderer(this, 0, 18);
        rightWing.func_78789_a(-3F, 1F, -3.5F, 7, 4, 1);
        rightWing.func_78793_a(0F, 12F, 0F);
        rightWing.field_78809_i = true;

        leftFoot = new ModelRenderer(this, 20, 3);
        leftFoot.func_78790_a(-2F, 11.9F, 0.5F, 4, 0, 3, 0.001F);
        leftFoot.func_78793_a(0F, 0F, 0F);
        leftFoot.field_78809_i = true;

        rightFoot = new ModelRenderer(this, 20, 3);
        rightFoot.func_78790_a(-2F, 11.9F, -3.5F, 4, 0, 3, 0.001F);
        rightFoot.func_78793_a(0F, 0F, 0F);
        rightFoot.field_78809_i = true;

        head.func_78792_a(neck);
        head.func_78792_a(chest);

        rightLeg.func_78792_a(rightLowerLeg);
        rightLowerLeg.func_78792_a(rightFoot);
        leftLeg.func_78792_a(leftLowerLeg);
        leftLowerLeg.func_78792_a(leftFoot);
    }

    @Override
    public void func_78088_a(Entity entity, float par2, float par3, float par4, float par5, float par6, float par7)
    {
        this.func_78087_a(par2, par3, par4, par5, par6, par7, entity);
        float percent = ((EntityPheasantTFC) entity).getPercentToAdulthood();
        float ageScale = 2.0F - percent;

        GlStateManager.func_179094_E();

        GlStateManager.func_179109_b(0.0F, 0.75f - (0.75f * percent), 0f);
        GlStateManager.func_179152_a(1 / ageScale, 1 / ageScale, 1 / ageScale);

        this.head.func_78785_a(par7);
        this.beak.func_78785_a(par7);

        GlStateManager.func_179121_F();

        GlStateManager.func_179094_E();
        GlStateManager.func_179109_b(0.0F, 0.75f - (0.75f * percent), 0f);
        GlStateManager.func_179152_a(1 / ageScale, 1 / ageScale, 1 / ageScale);
        this.body.func_78785_a(par7);
        this.rightLeg.func_78785_a(par7);
        this.leftLeg.func_78785_a(par7);
        this.rightWing.func_78785_a(par7);
        this.leftWing.func_78785_a(par7);
        this.tail.func_78785_a(par7);
        this.tailFeather.func_78785_a(par7);
        GlStateManager.func_179121_F();
    }

    @Override
    public void func_78087_a(float par1, float par2, float par3, float par4, float par5, float par6, Entity entity)
    {
        this.head.field_78796_g = par4 / (180F / (float) Math.PI);
        this.beak.field_78796_g = this.head.field_78796_g;
        this.neck.field_78808_h = -(5 * (float) (Math.PI / 18F));
        this.chest.field_78808_h = -(8 * (float) (Math.PI / 18F));
        this.body.field_78808_h = -((float) (Math.PI / 6F));
        this.rightWing.field_78808_h = -((float) (Math.PI / 6F));
        this.leftWing.field_78808_h = -((float) (Math.PI / 6F));
        if (par3 != 0)
        {
            rightWing.func_78793_a(4, 12, -2);
            leftWing.func_78793_a(4, 12, 2);
            rightWing.field_78808_h = -(float) (Math.PI / 2F);
            leftWing.field_78808_h = -(float) (Math.PI / 2F);
            rightWing.field_82906_o = -3F / 16F;
            rightWing.field_82908_p = -3F / 16F;
            rightWing.field_82907_q = -1.5F / 16F;
            leftWing.field_82906_o = -3F / 16F;
            leftWing.field_82908_p = -3F / 16F;
            leftWing.field_82907_q = 1.5F / 16F;
        }
        else
        {
            rightWing.func_78793_a(0, 12, 0);
            leftWing.func_78793_a(0, 12, 0);
            rightWing.field_82906_o = 0;
            rightWing.field_82908_p = 0;
            rightWing.field_82907_q = 0;
            leftWing.field_82906_o = 0;
            leftWing.field_82908_p = 0;
            leftWing.field_82907_q = 0;
        }
        this.rightWing.field_78796_g = par3;
        this.leftWing.field_78796_g = -par3;


        this.tail.field_78808_h = -((float) (Math.PI / 18F));
        this.tailFeather.field_78808_h = -((float) (Math.PI / 36F));
        this.rightLeg.field_78808_h = ((float) (Math.PI / 9F)) + MathHelper.func_76134_b(par1 * 0.6662F) * 1.4F * par2;
        this.leftLeg.field_78808_h = ((float) (Math.PI / 9F)) + MathHelper.func_76134_b(par1 * 0.6662F + (float) Math.PI) * 1.4F * par2;
        this.rightLowerLeg.field_78808_h = -((float) (Math.PI / 9F));
        this.leftLowerLeg.field_78808_h = -((float) (Math.PI / 9F));
        this.rightFoot.field_78808_h = 0;
        this.leftFoot.field_78808_h = 0;
    }

}
