/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.render.animal;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

import org.lwjgl.opengl.GL11;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.entity.RenderManager;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.client.event.RenderLivingEvent;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

@SideOnly(Side.CLIENT)
public final class RenderAnimalTFCFamiliarity
{
	private static final ResourceLocation ICONS = new ResourceLocation(MOD_ID, "textures/gui/overlay/icons.png");
	private static final RenderAnimalTFCFamiliarity INSTANCE = new RenderAnimalTFCFamiliarity();

	public static RenderAnimalTFCFamiliarity getInstance() { return INSTANCE; }

	@SuppressWarnings("PointlessArithmeticExpression")
	public void drawTexturedModalRect(float xCoord, float yCoord, int minU, int minV, int maxU, int maxV)
	{
		float textureScaleU = 0.00390625F;
		float textureScaleV = 0.00390625F;
		Tessellator tessellator = Tessellator.func_178181_a();
		BufferBuilder vb = tessellator.func_178180_c();
		vb.func_181668_a(7, DefaultVertexFormats.field_181707_g);
		vb.func_181662_b(xCoord + 0.0F, yCoord + maxV, 0).func_187315_a((minU + 0) * textureScaleU, (minV + maxV) * textureScaleV)
				.func_181675_d();
		vb.func_181662_b(xCoord + maxU, yCoord + maxV, 0).func_187315_a((minU + maxU) * textureScaleU, (minV + maxV) * textureScaleV)
				.func_181675_d();
		vb.func_181662_b(xCoord + maxU, yCoord + 0.0F, 0).func_187315_a((minU + maxU) * textureScaleU, (minV + 0) * textureScaleV)
				.func_181675_d();
		vb.func_181662_b(xCoord + 0.0F, yCoord + 0.0F, 0).func_187315_a((minU + 0) * textureScaleU, (minV + 0) * textureScaleV).func_181675_d();
		tessellator.func_78381_a();
	}

	@SubscribeEvent(priority = EventPriority.LOW)
	public void renderAnimalFamiliarity(RenderLivingEvent.Post<EntityAnimalTFC> event)
	{
		Minecraft mc = Minecraft.func_71410_x();
		EntityPlayer player = mc.field_71439_g.field_71071_by.field_70458_d;

		if (player.func_70093_af())
		{
			EntityLivingBase entity = event.getEntity();
			if (entity instanceof EntityAnimalTFC)
			{
				double x, y, z;
				x = event.getX();
				y = event.getY();
				z = event.getZ();

				float f = 1.6F;
				float f1 = 0.016666668F * f;
				double d3 = entity.func_70032_d(player);
				float f2 = 5.0F;

				if (d3 < f2)
				{
					EntityAnimalTFC animal = (EntityAnimalTFC) entity;
					RenderManager rendermanager = mc.func_175598_ae();

					GL11.glPushMatrix();
					GL11.glTranslatef((float) x + 0.0F, (float) y + entity.field_70131_O + 0.75F, (float) z);
					GL11.glRotatef(-rendermanager.field_78735_i, 0.0F, 1.0F, 0.0F);
					GL11.glRotatef(rendermanager.field_78732_j, 1.0F, 0.0F, 0.0F);
					GL11.glScalef(-f1, -f1, f1);
					GL11.glDisable(GL11.GL_LIGHTING);
					GL11.glTranslatef(0.0F, 0.25F / f1, 0.0F);
					GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
					mc.field_71446_o.func_110577_a(ICONS);
					GL11.glScalef(0.33F, 0.33F, 0.33F);

					float familiarity = Math.max(0.0F, Math.min(1.0F, animal.getFamiliarity()));
					if (familiarity >= 0.3F) {
						this.drawTexturedModalRect(-8, 0, 112, 40, 16, 16);
					} else {
						this.drawTexturedModalRect(-8, 0, 92, 40, 16, 16);
					}

					GL11.glTranslatef(0, 0, -0.001F);

					if (familiarity == 1.0F) {
						this.drawTexturedModalRect(-6, 14 - (int) (12 * familiarity), 114, 74 - (int) (12 * familiarity), 12, (int) (12 * familiarity));
					} else {
						this.drawTexturedModalRect(-6, 14 - (int) (12 * familiarity), 94, 74 - (int) (12 * familiarity), 12, (int) (12 * familiarity));
					}

					GL11.glDepthMask(true);
					GL11.glEnable(GL11.GL_LIGHTING);
					GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
					GL11.glPopMatrix();
				}
			}
		}
	}
}
