/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.command;

import java.util.Random;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.command.CommandBase;
import net.minecraft.command.CommandException;
import net.minecraft.command.ICommandSender;
import net.minecraft.command.WrongUsageException;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.WorldServer;
import net.minecraft.world.gen.structure.template.TemplateManager;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.registries.TFCRegistries;
import net.dries007.tfc.api.types.Tree;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class CommandGenTree extends CommandBase
{
    private static final Random random = new Random();

    @Override
    public String func_71517_b()
    {
        return "maketree";
    }

    @Override
    public String func_71518_a(ICommandSender sender)
    {
        return "/maketree [wood] -> Grows a tree of the type specified";
    }

    @Override
    public void func_184881_a(MinecraftServer server, ICommandSender sender, String[] args) throws CommandException
    {
        if (args.length != 1) throw new WrongUsageException("1 argument required.");

        Tree tree = TFCRegistries.TREES.getValue(new ResourceLocation(args[0]));
        if (tree == null) tree = TFCRegistries.TREES.getValue(new ResourceLocation(MOD_ID, args[0]));
        if (tree == null) throw new WrongUsageException("Tree type " + args[0] + " not found");

        if (sender.func_174793_f() == null) return;

        final World world = sender.func_130014_f_();
        final BlockPos center = new BlockPos(sender.func_174793_f());
        final TemplateManager manager = ((WorldServer) world).func_184163_y();

        tree.makeTreeWithoutChecking(manager, world, center, random);
    }

    @Override
    public int func_82362_a()
    {
        return 2;
    }
}
