/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.network;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.network.simpleimpl.IMessage;
import net.minecraftforge.fml.common.network.simpleimpl.IMessageHandler;
import net.minecraftforge.fml.common.network.simpleimpl.MessageContext;

import io.netty.buffer.ByteBuf;
import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.recipes.barrel.BarrelRecipe;
import net.dries007.tfc.api.registries.TFCRegistries;
import net.dries007.tfc.objects.te.TEBarrel;
import net.dries007.tfc.util.Helpers;

public class PacketBarrelUpdate implements IMessage
{
    private BlockPos pos;
    private long calendarTick;
    private ResourceLocation recipeName;

    @SuppressWarnings("unused")
    @Deprecated
    public PacketBarrelUpdate() {}

    public PacketBarrelUpdate(@Nonnull TEBarrel tile, @Nullable BarrelRecipe currentRecipe, long calendarTick)
    {
        this.pos = tile.func_174877_v();
        this.calendarTick = calendarTick;
        this.recipeName = currentRecipe != null ? currentRecipe.getRegistryName() : null;
    }

    @Override
    public void fromBytes(ByteBuf buf)
    {
        pos = BlockPos.func_177969_a(buf.readLong());
        calendarTick = buf.readLong();
        recipeName = Helpers.readResourceLocation(buf);
    }

    @Override
    public void toBytes(ByteBuf buf)
    {
        buf.writeLong(pos.func_177986_g());
        buf.writeLong(calendarTick);
        Helpers.writeResourceLocation(buf, recipeName);
    }

    public static final class Handler implements IMessageHandler<PacketBarrelUpdate, IMessage>
    {
        @Override
        public IMessage onMessage(PacketBarrelUpdate message, MessageContext ctx)
        {
            EntityPlayer player = TerraFirmaCraft.getProxy().getPlayer(ctx);
            if (player != null)
            {
                World world = player.func_130014_f_();
                TerraFirmaCraft.getProxy().getThreadListener(ctx).func_152344_a(() -> {
                    TEBarrel te = Helpers.getTE(world, message.pos, TEBarrel.class);
                    if (te != null)
                    {
                        BarrelRecipe recipe = message.recipeName == null ? null : TFCRegistries.BARREL.getValue(message.recipeName);
                        te.onReceivePacket(recipe, message.calendarTick);
                    }
                });
            }
            return null;
        }
    }
}
