/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.network;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.network.ByteBufUtils;
import net.minecraftforge.fml.common.network.simpleimpl.IMessage;
import net.minecraftforge.fml.common.network.simpleimpl.IMessageHandler;
import net.minecraftforge.fml.common.network.simpleimpl.MessageContext;

import io.netty.buffer.ByteBuf;
import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.objects.te.TECrucible;
import net.dries007.tfc.util.Helpers;

/**
 * Used to send the alloy contents of a crucible to client for display purposes
 */
public class PacketCrucibleUpdate implements IMessage
{
    private NBTTagCompound alloyNBT;
    private BlockPos pos;

    @SuppressWarnings("unused")
    @Deprecated
    public PacketCrucibleUpdate() {}

    public PacketCrucibleUpdate(TECrucible tile)
    {
        this.pos = tile.func_174877_v();
        this.alloyNBT = tile.getAlloy().serializeNBT();
    }

    @Override
    public void fromBytes(ByteBuf buf)
    {
        pos = BlockPos.func_177969_a(buf.readLong());
        alloyNBT = ByteBufUtils.readTag(buf);
    }

    @Override
    public void toBytes(ByteBuf buf)
    {
        buf.writeLong(pos.func_177986_g());
        ByteBufUtils.writeTag(buf, alloyNBT);
    }

    public static class Handler implements IMessageHandler<PacketCrucibleUpdate, IMessage>
    {
        @Override
        public IMessage onMessage(PacketCrucibleUpdate message, MessageContext ctx)
        {
            EntityPlayer player = TerraFirmaCraft.getProxy().getPlayer(ctx);
            if (player != null)
            {
                World world = player.func_130014_f_();
                TerraFirmaCraft.getProxy().getThreadListener(ctx).func_152344_a(() -> {
                    TECrucible te = Helpers.getTE(world, message.pos, TECrucible.class);
                    if (te != null)
                    {
                        te.setAlloy(message.alloyNBT);
                    }
                });
            }
            return null;
        }
    }
}
