/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.agriculture;

import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.dries007.tfc.api.types.ICrop;
import net.dries007.tfc.objects.items.ItemSeedsTFC;
import net.dries007.tfc.util.Helpers;

@ParametersAreNonnullByDefault
public abstract class BlockCropSimple extends BlockCropTFC
{
    private static final AxisAlignedBB[] CROPS_AABB = new AxisAlignedBB[] {
        new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 0.125D, 0.875D),
        new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 0.25D, 0.875D),
        new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 0.375D, 0.875D),
        new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 0.5D, 0.875D),
        new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 0.625D, 0.875D),
        new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 0.75D, 0.875D),
        new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 0.875D, 0.875D),
        new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 1.0D, 0.875D)
    };

    private final boolean isPickable;

    protected BlockCropSimple(ICrop crop, boolean isPickable)
    {
        super(crop);
        this.isPickable = isPickable;

        func_180632_j(func_176194_O().func_177621_b().func_177226_a(getStageProperty(), 0).func_177226_a(WILD, false));
    }

    @Override
    public void func_176474_b(World worldIn, Random rand, BlockPos pos, IBlockState state)
    {
        if (!worldIn.field_72995_K)
        {
            if (state.func_177229_b(getStageProperty()) < crop.getMaxStage())
            {
                worldIn.func_180501_a(pos, state.func_177226_a(getStageProperty(), state.func_177229_b(getStageProperty()) + 1), 2);
            }
        }
    }

    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public IBlockState func_176203_a(int meta)
    {
        return this.func_176223_P().func_177226_a(getStageProperty(), meta & 7).func_177226_a(WILD, meta > 7);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(getStageProperty()) + (state.func_177229_b(WILD) ? 8 : 0);
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        if (isPickable)
        {
            ItemStack foodDrop = crop.getFoodDrop(state.func_177229_b(getStageProperty()));
            if (!foodDrop.func_190926_b())
            {
                if (!worldIn.field_72995_K)
                {
                    worldIn.func_175656_a(pos, this.func_176223_P().func_177226_a(getStageProperty(), state.func_177229_b(getStageProperty()) - 2));
                    Helpers.spawnItemStack(worldIn, pos, crop.getFoodDrop(crop.getMaxStage()));
                }
                return true;
            }
        }
        return false;
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, getStageProperty(), WILD);
    }

    @Override
    public void getDrops(NonNullList<ItemStack> drops, IBlockAccess world, BlockPos pos, IBlockState state, int fortune)
    {
        drops.clear();
        drops.add(new ItemStack(ItemSeedsTFC.get(crop)));

        // todo: adjust food drops based on player agriculture skill. For now just go with 2 for initial balance
        ItemStack foodDrop = crop.getFoodDrop(state.func_177229_b(getStageProperty()));
        if (!foodDrop.func_190926_b())
        {
            foodDrop.func_190920_e(2);
            drops.add(foodDrop);
        }
    }

    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return CROPS_AABB[state.func_177229_b(getStageProperty())];
    }
}
