/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.devices;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.particle.ParticleManager;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumBlockRenderType;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraft.world.WorldServer;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.objects.blocks.property.ILightableBlock;
import net.dries007.tfc.objects.items.ItemsTFC;
import net.dries007.tfc.objects.te.TEPitKiln;
import net.dries007.tfc.util.Helpers;

import static net.dries007.tfc.objects.blocks.BlockPlacedItem.PLACED_ITEM_AABB;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class BlockPitKiln extends Block implements ILightableBlock
{
    public static final PropertyBool FULL = PropertyBool.func_177716_a("full");

    private static final AxisAlignedBB[] AABB_LEVELS = new AxisAlignedBB[] {
        PLACED_ITEM_AABB,
        new AxisAlignedBB(0, 0, 0, 1, 0.0625, 1),
        new AxisAlignedBB(0, 0, 0, 1, 0.125, 1),
        new AxisAlignedBB(0, 0, 0, 1, 0.1875, 1),
        new AxisAlignedBB(0, 0, 0, 1, 0.25, 1),
        new AxisAlignedBB(0, 0, 0, 1, 0.3125, 1),
        new AxisAlignedBB(0, 0, 0, 1, 0.375, 1),
        new AxisAlignedBB(0, 0, 0, 1, 0.4375, 1),
        new AxisAlignedBB(0, 0, 0, 1, 0.5, 1),
        new AxisAlignedBB(0, 0, 0, 1, 0.75, 1),
        field_185505_j
    };

    public BlockPitKiln()
    {
        super(Material.field_151594_q);
        func_149711_c(0.5f);
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(FULL, false).func_177226_a(LIT, false));
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_185481_k(IBlockState state)
    {
        // This is required for the fire code, because forge doesn't 'fix' it to use the location sensitive version.
        return true;
    }

    @Override
    @SuppressWarnings("deprecation")
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P();
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return 0;
    }

    @Override
    @SuppressWarnings("deprecation")
    public IBlockState func_176221_a(IBlockState state, IBlockAccess worldIn, BlockPos pos)
    {
        TEPitKiln te = Helpers.getTE(worldIn, pos, TEPitKiln.class);
        if (te != null)
        {
            return state.func_177226_a(BlockPitKiln.LIT, te.isLit()).func_177226_a(BlockPitKiln.FULL, te.hasFuel());
        }
        return state;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return true;
    }

    @Override
    @SuppressWarnings("deprecation")
    public EnumBlockRenderType func_149645_b(IBlockState state)
    {
        return EnumBlockRenderType.ENTITYBLOCK_ANIMATED;
    }

    @Override
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        TEPitKiln tile = Helpers.getTE(source, pos, TEPitKiln.class);
        if (tile != null)
        {
            int height = tile.getStrawCount();
            if (tile.getLogCount() > 4)
            {
                height = 10; // Full block
            }
            else if (tile.getLogCount() > 0)
            {
                height = 9; // 75% of block
            }
            return AABB_LEVELS[height];
        }
        return PLACED_ITEM_AABB;
    }

    @Override
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return BlockFaceShape.UNDEFINED;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        TEPitKiln te = Helpers.getTE(worldIn, pos, TEPitKiln.class);
        if (te != null)
        {
            if (blockIn == Blocks.field_150480_ab)
            {
                te.tryLight();
            }
            // Make sure the sides are valid
            te.assertValid();
        }
        super.func_189540_a(state, worldIn, pos, blockIn, fromPos);
    }

    @Override
    public void func_180663_b(World worldIn, BlockPos pos, IBlockState state)
    {
        TEPitKiln te = Helpers.getTE(worldIn, pos, TEPitKiln.class);
        if (te != null)
        {
            te.onBreakBlock(worldIn, pos);
        }
        super.func_180663_b(worldIn, pos, state);
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        TEPitKiln te = Helpers.getTE(worldIn, pos, TEPitKiln.class);
        if (te != null)
        {
            // Skip interacting if using a fire starter
            if (playerIn.func_184614_ca().func_77973_b() == ItemsTFC.FIRESTARTER || playerIn.func_184592_cb().func_77973_b() == ItemsTFC.FIRESTARTER)
            {
                return false;
            }
            return te.onRightClick(playerIn, playerIn.func_184586_b(hand), hitX < 0.5, hitZ < 0.5);
        }
        return false;
    }

    @Override
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, FULL, LIT);
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean isSideSolid(IBlockState state, IBlockAccess world, BlockPos pos, EnumFacing side)
    {
        return state.func_185899_b(world, pos).func_177229_b(FULL);
    }

    @Override
    public boolean isBurning(IBlockAccess world, BlockPos pos)
    {
        return world.func_180495_p(pos).func_185899_b(world, pos).func_177229_b(LIT);
    }

    @Override
    public int getFlammability(IBlockAccess world, BlockPos pos, EnumFacing face)
    {
        return 0;
    }

    @Override
    public int getFireSpreadSpeed(IBlockAccess world, BlockPos pos, EnumFacing face)
    {
        return world.func_180495_p(pos).func_185899_b(world, pos).func_177229_b(LIT) ? 120 : 0; // Twice as much as the highest vanilla level (60)
    }

    @Override
    public boolean isFireSource(World world, BlockPos pos, EnumFacing side)
    {
        return world.func_180495_p(pos).func_185899_b(world, pos).func_177229_b(LIT);
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TEPitKiln();
    }

    @Override
    public boolean addLandingEffects(IBlockState state, WorldServer worldObj, BlockPos blockPosition, IBlockState iblockstate, EntityLivingBase entity, int numberOfParticles)
    {
        return true;
    }

    @Override
    public boolean addRunningEffects(IBlockState state, World world, BlockPos pos, Entity entity)
    {
        return true;
    }

    @SideOnly(Side.CLIENT)
    @Override
    public boolean addHitEffects(IBlockState state, World worldObj, RayTraceResult target, ParticleManager manager)
    {
        return true;
    }

    @SideOnly(Side.CLIENT)
    @Override
    public boolean addDestroyEffects(World world, BlockPos pos, ParticleManager manager)
    {
        return true;
    }
}
