/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.plants;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.IGrowable;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.dries007.tfc.api.types.Plant;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

@ParametersAreNonnullByDefault
public class BlockHangingPlantTFC extends BlockCreepingPlantTFC implements IGrowable
{
    private static final PropertyBool BOTTOM = PropertyBool.func_177716_a("bottom");
    private static final Map<Plant, BlockHangingPlantTFC> MAP = new HashMap<>();

    public static BlockHangingPlantTFC get(Plant plant)
    {
        return BlockHangingPlantTFC.MAP.get(plant);
    }

    public BlockHangingPlantTFC(Plant plant)
    {
        super(plant);
        if (MAP.put(plant, this) != null) throw new IllegalStateException("There can only be one.");
    }

    @Override
    public boolean func_176473_a(World worldIn, BlockPos pos, IBlockState state, boolean isClient)
    {
        IBlockState iblockstate = worldIn.func_180495_p(pos.func_177979_c(2));
        Material material = iblockstate.func_185904_a();

        int i;
        //noinspection StatementWithEmptyBody
        for (i = 1; worldIn.func_180495_p(pos.func_177981_b(i)).func_177230_c() == this; ++i) ;
        return i < plant.getMaxHeight() && worldIn.func_175623_d(pos.func_177977_b()) && ((!material.func_76220_a() || material == Material.field_151584_j)) && func_180671_f(worldIn, pos.func_177977_b(), state);
    }

    @Override
    public boolean func_180670_a(World worldIn, Random rand, BlockPos pos, IBlockState state)
    {
        return false;
    }

    @Override
    public void func_176474_b(World worldIn, Random rand, BlockPos pos, IBlockState state)
    {
        worldIn.func_175656_a(pos.func_177977_b(), this.func_176223_P());
        IBlockState iblockstate = state.func_177226_a(AGE, 0).func_177226_a(growthStageProperty, plant.getStageForMonth()).func_177226_a(BOTTOM, false);
        worldIn.func_175656_a(pos, iblockstate);
        iblockstate.func_189546_a(worldIn, pos.func_177977_b(), this, pos);
    }

    public void shrink(World worldIn, BlockPos pos)
    {
        worldIn.func_175698_g(pos);
        worldIn.func_180495_p(pos).func_189546_a(worldIn, pos.func_177984_a(), this, pos);
    }

    @Override
    @Nonnull
    public IBlockState func_176221_a(IBlockState state, IBlockAccess worldIn, BlockPos pos)
    {
        IBlockState actualState = super.func_176221_a(state, worldIn, pos);
        if (worldIn.func_180495_p(pos.func_177977_b()).func_177230_c() == this && actualState.func_177229_b(UP) && !actualState.func_177229_b(DOWN) && !actualState.func_177229_b(NORTH) && !actualState.func_177229_b(SOUTH) && !actualState.func_177229_b(EAST) && !actualState.func_177229_b(WEST))
        {
            actualState = actualState.func_177226_a(NORTH, true).func_177226_a(SOUTH, true).func_177226_a(EAST, true).func_177226_a(WEST, true);
        }
        if (worldIn.func_180495_p(pos.func_177984_a()).func_177230_c() == this && !actualState.func_177229_b(UP) && !actualState.func_177229_b(NORTH) && !actualState.func_177229_b(SOUTH) && !actualState.func_177229_b(EAST) && !actualState.func_177229_b(WEST))
        {
            if (!actualState.func_177229_b(DOWN))
            {
                actualState = actualState.func_185899_b(worldIn, pos.func_177984_a()).func_177226_a(UP, false);
            }
            else
            {
                actualState = actualState.func_185899_b(worldIn, pos.func_177984_a()).func_177226_a(DOWN, true).func_177226_a(UP, false);
            }
        }
        return actualState.func_177226_a(BOTTOM, getIsBottom(worldIn, pos));
    }

    @Override
    public boolean func_180671_f(World worldIn, BlockPos pos, IBlockState state)
    {
        for (EnumFacing face : EnumFacing.values())
        {
            IBlockState blockState = worldIn.func_180495_p(pos.func_177972_a(face));
            Material material = blockState.func_185904_a();

            if (material == Material.field_151584_j || worldIn.func_180495_p(pos.func_177984_a()).func_177230_c() == this)
            {
                return plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) && plant.isValidRain(ChunkDataTFC.getRainfall(worldIn, pos));
            }
        }
        return false;
    }

    @Override
    @Nonnull
    protected BlockStateContainer createPlantBlockState()
    {
        return new BlockStateContainer(this, DOWN, UP, NORTH, EAST, WEST, SOUTH, growthStageProperty, DAYPERIOD, AGE, BOTTOM);
    }

    @Override
    protected boolean canConnectTo(IBlockAccess worldIn, BlockPos pos, EnumFacing facing)
    {
        IBlockState iblockstate = worldIn.func_180495_p(pos);
        Material material = iblockstate.func_185904_a();

        return material == Material.field_151584_j;
    }

    @Override
    protected boolean canPlantConnectTo(IBlockAccess world, BlockPos pos, EnumFacing facing)
    {
        if (!super.canPlantConnectTo(world, pos, facing) && world.func_180495_p(pos.func_177984_a()).func_177230_c() == this && facing != EnumFacing.DOWN && facing != EnumFacing.UP)
        {
            return canPlantConnectTo(world, pos.func_177984_a(), facing);
        }

        return super.canPlantConnectTo(world, pos, facing);
    }

    @Override
    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand)
    {
        if (!worldIn.func_175697_a(pos, 1)) return;

        if (plant.isValidGrowthTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) && plant.isValidSunlight(Math.subtractExact(worldIn.func_175642_b(EnumSkyBlock.SKY, pos), worldIn.func_175657_ab())))
        {
            int j = state.func_177229_b(AGE);

            if (rand.nextDouble() < getGrowthRate(worldIn, pos) && net.minecraftforge.common.ForgeHooks.onCropsGrowPre(worldIn, pos.func_177977_b(), state, true))
            {
                if (j == 3)
                {
                    if (func_176473_a(worldIn, pos, state, worldIn.field_72995_K)) func_176474_b(worldIn, rand, pos, state);
                    else if (canGrowHorizontally(worldIn, pos, state)) growHorizontally(worldIn, rand, pos, state);
                    else if (canGrowDiagonally(worldIn, pos, state)) growDiagonally(worldIn, rand, pos, state);
                }
                else if (j < 3)
                {
                    worldIn.func_175656_a(pos, state.func_177226_a(AGE, j + 1).func_177226_a(BOTTOM, getIsBottom(worldIn, pos)));
                }
                net.minecraftforge.common.ForgeHooks.onCropsGrowPost(worldIn, pos, state, worldIn.func_180495_p(pos));
            }
        }
        else if (!plant.isValidGrowthTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) || !plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, pos)))
        {
            int j = state.func_177229_b(AGE);

            if (rand.nextDouble() < getGrowthRate(worldIn, pos) && net.minecraftforge.common.ForgeHooks.onCropsGrowPre(worldIn, pos, state, true))
            {
                if (j == 0)
                {
                    if (canShrink(worldIn, pos)) shrink(worldIn, pos);
                    else if (canShrinkHorizontally(worldIn, pos)) shrinkHorizontally(worldIn, pos);
                }
                else if (j > 0)
                {
                    worldIn.func_175656_a(pos, state.func_177226_a(AGE, j - 1).func_177226_a(BOTTOM, getIsBottom(worldIn, pos)));
                }
                net.minecraftforge.common.ForgeHooks.onCropsGrowPost(worldIn, pos, state, worldIn.func_180495_p(pos));
            }
        }

        func_176475_e(worldIn, pos, state);
    }

    private boolean canGrowDiagonally(World worldIn, BlockPos pos, IBlockState state)
    {
        boolean flag = false;
        if (!state.func_177229_b(BOTTOM))
        {
            for (EnumFacing face : EnumFacing.Plane.HORIZONTAL.func_179516_a())
            {
                BlockPos sidePos = pos.func_177972_a(face);
                IBlockState sideState = worldIn.func_180495_p(sidePos.func_177979_c(2));
                Material sideMaterial = sideState.func_185904_a();

                if (worldIn.func_175623_d(sidePos) && worldIn.func_175623_d(sidePos.func_177977_b()) && (!sideMaterial.func_76220_a() || sideMaterial == Material.field_151584_j) && func_180671_f(worldIn, sidePos.func_177977_b(), state))
                {
                    flag = true;
                }
            }
        }
        return flag;
    }

    private void growDiagonally(World worldIn, Random rand, BlockPos pos, IBlockState state)
    {
        if (!state.func_177229_b(BOTTOM))
        {
            for (EnumFacing face : EnumFacing.Plane.HORIZONTAL.func_179516_a())
            {
                BlockPos sidePos = pos.func_177972_a(face);

                if (rand.nextDouble() < 0.5D && worldIn.func_175623_d(sidePos) && worldIn.func_175623_d(sidePos.func_177977_b()))
                {
                    worldIn.func_175656_a(sidePos.func_177977_b(), this.func_176223_P());
                    IBlockState iblockstate = state.func_177226_a(AGE, 0).func_177226_a(growthStageProperty, plant.getStageForMonth());
                    worldIn.func_175656_a(pos, iblockstate);
                    iblockstate.func_189546_a(worldIn, sidePos.func_177977_b(), this, pos);
                    break;
                }
            }
        }
    }

    private boolean canGrowHorizontally(World worldIn, BlockPos pos, IBlockState state)
    {
        boolean flag = false;
        for (EnumFacing face : EnumFacing.Plane.HORIZONTAL.func_179516_a())
        {
            BlockPos sidePos = pos.func_177972_a(face);
            IBlockState sideState = worldIn.func_180495_p(sidePos.func_177977_b());
            Material sideMaterial = sideState.func_185904_a();

            if (worldIn.func_175623_d(sidePos) && (!sideMaterial.func_76220_a() || sideMaterial == Material.field_151584_j) && func_180671_f(worldIn, sidePos, state))
            {
                flag = true;
            }
        }
        return flag;
    }

    private void growHorizontally(World worldIn, Random rand, BlockPos pos, IBlockState state)
    {
        for (EnumFacing face : EnumFacing.Plane.HORIZONTAL.func_179516_a())
        {
            BlockPos sidePos = pos.func_177972_a(face);

            if (rand.nextDouble() < 0.01D && worldIn.func_175623_d(sidePos))
            {
                worldIn.func_175656_a(sidePos, this.func_176223_P());
                IBlockState iblockstate = state.func_177226_a(AGE, 0).func_177226_a(growthStageProperty, plant.getStageForMonth());
                worldIn.func_175656_a(pos, iblockstate);
                iblockstate.func_189546_a(worldIn, sidePos, this, pos);
                break;
            }
        }
    }

    private void shrinkHorizontally(World worldIn, BlockPos pos)
    {
        worldIn.func_175698_g(pos);
        IBlockState state = worldIn.func_180495_p(pos);
        state.func_189546_a(worldIn, pos.func_177974_f(), this, pos);
        state.func_189546_a(worldIn, pos.func_177976_e(), this, pos);
        state.func_189546_a(worldIn, pos.func_177978_c(), this, pos);
        state.func_189546_a(worldIn, pos.func_177968_d(), this, pos);
    }

    private boolean canShrink(World worldIn, BlockPos pos)
    {
        return worldIn.func_180495_p(pos.func_177984_a()).func_177230_c() == this && worldIn.func_180495_p(pos.func_177977_b()).func_177230_c() != this;
    }

    private boolean canShrinkHorizontally(World worldIn, BlockPos pos)
    {
        boolean flag = false;
        for (EnumFacing face : EnumFacing.Plane.HORIZONTAL.func_179516_a())
        {
            if (worldIn.func_180495_p(pos.func_177972_a(face)).func_177230_c() == this)
            {
                flag = true;
            }
        }
        return flag;
    }

    private boolean getIsBottom(IBlockAccess world, BlockPos pos)
    {
        IBlockState iblockstate = world.func_180495_p(pos.func_177977_b());
        Material material = iblockstate.func_185904_a();

        return world.func_180495_p(pos.func_177977_b()).func_177230_c() != this && !material.func_76220_a();
    }
}
