/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.plants;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.IShearable;

import net.dries007.tfc.api.types.Plant;
import net.dries007.tfc.objects.items.ItemsTFC;
import net.dries007.tfc.world.classic.ClimateTFC;

@ParametersAreNonnullByDefault
public class BlockShortGrassTFC extends BlockPlantTFC implements IShearable
{
    private static final AxisAlignedBB GRASS_AABB = new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 1.0D, 0.875D);
    private static final AxisAlignedBB SHORTER_GRASS_AABB = new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 0.5D, 0.875D);
    private static final AxisAlignedBB SHORT_GRASS_AABB = new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 0.75D, 0.875D);
    private static final AxisAlignedBB SHORTEST_GRASS_AABB = new AxisAlignedBB(0.125D, 0.0D, 0.125D, 0.875D, 0.25D, 0.875D);
    private static final Map<Plant, BlockShortGrassTFC> MAP = new HashMap<>();

    public static BlockShortGrassTFC get(Plant plant)
    {
        return BlockShortGrassTFC.MAP.get(plant);
    }

    public BlockShortGrassTFC(Plant plant)
    {
        super(plant);
        if (MAP.put(plant, this) != null) throw new IllegalStateException("There can only be one.");
    }

    @Override
    public void func_180657_a(World worldIn, EntityPlayer player, BlockPos pos, IBlockState state, @Nullable TileEntity te, ItemStack stack)
    {
        if (!worldIn.field_72995_K && stack.func_77973_b() == Items.field_151097_aZ)
        {
            func_180635_a(worldIn, pos, new ItemStack(this, 1));
        }
        else if (!worldIn.field_72995_K && stack.func_77973_b().getHarvestLevel(stack, "knife", player, state) != -1 || stack.func_77973_b().getHarvestLevel(stack, "scythe", player, state) != -1)
        {
            func_180635_a(worldIn, pos, new ItemStack(ItemsTFC.STRAW, 1));
        }
        super.func_180657_a(worldIn, player, pos, state, te, stack);
    }

    @Override
    @Nonnull
    public Block.EnumOffsetType func_176218_Q()
    {
        return Block.EnumOffsetType.XZ;
    }

    @Override
    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand)
    {
        if (!worldIn.func_175697_a(pos, 1)) return;

        if (plant.isValidGrowthTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) && plant.isValidSunlight(Math.subtractExact(worldIn.func_175642_b(EnumSkyBlock.SKY, pos), worldIn.func_175657_ab())))
        {
            int j = state.func_177229_b(AGE);

            if (rand.nextDouble() < getGrowthRate(worldIn, pos) && net.minecraftforge.common.ForgeHooks.onCropsGrowPre(worldIn, pos.func_177984_a(), state, true))
            {
                if (j < 3)
                {
                    worldIn.func_175656_a(pos, state.func_177226_a(AGE, j + 1));
                }
                net.minecraftforge.common.ForgeHooks.onCropsGrowPost(worldIn, pos, state, worldIn.func_180495_p(pos));
            }
        }
        else if (!plant.isValidGrowthTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) || !plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, pos)))
        {
            int j = state.func_177229_b(AGE);

            if (rand.nextDouble() < getGrowthRate(worldIn, pos) && net.minecraftforge.common.ForgeHooks.onCropsGrowPre(worldIn, pos, state, true))
            {
                if (j > 0)
                {
                    worldIn.func_175656_a(pos, state.func_177226_a(AGE, j - 1));
                }
                else
                {
                    worldIn.func_175698_g(pos);
                }
                net.minecraftforge.common.ForgeHooks.onCropsGrowPost(worldIn, pos, state, worldIn.func_180495_p(pos));
            }
        }

        func_176475_e(worldIn, pos, state);
    }

    @Override
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        switch (state.func_177229_b(AGE))
        {
            case 0:
                return SHORTEST_GRASS_AABB.func_191194_a(state.func_191059_e(source, pos));
            case 1:
                return SHORTER_GRASS_AABB.func_191194_a(state.func_191059_e(source, pos));
            case 2:
                return SHORT_GRASS_AABB.func_191194_a(state.func_191059_e(source, pos));
            default:
                return GRASS_AABB.func_191194_a(state.func_191059_e(source, pos));
        }
    }

    @Override
    @Nonnull
    protected BlockStateContainer createPlantBlockState()
    {
        return new BlockStateContainer(this, AGE, growthStageProperty, DAYPERIOD);
    }

    @Override
    public int func_149679_a(int fortune, Random random)
    {
        return 1 + random.nextInt(fortune * 2 + 1);
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public ItemStack func_185473_a(World worldIn, BlockPos pos, IBlockState state)
    {
        return new ItemStack(this, 1);
    }

    @Override
    @Nonnull
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player)
    {
        return new ItemStack(this, 1);
    }

    @Override
    public boolean isShearable(ItemStack item, IBlockAccess world, BlockPos pos)
    {
        return true;
    }

    @Override
    @Nonnull
    public NonNullList<ItemStack> onSheared(ItemStack item, IBlockAccess world, BlockPos pos, int fortune)
    {
        return NonNullList.func_191197_a(1, new ItemStack(this, 1));
    }
}
