/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.entity.animal;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.*;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.*;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.World;
import net.minecraft.world.biome.Biome;
import net.minecraftforge.oredict.OreDictionary;

import net.dries007.tfc.Constants;
import net.dries007.tfc.api.capability.egg.CapabilityEgg;
import net.dries007.tfc.api.capability.egg.IEgg;
import net.dries007.tfc.client.TFCSoundEvents;
import net.dries007.tfc.objects.LootTablesTFC;
import net.dries007.tfc.objects.entity.ai.EntityAIFindNest;
import net.dries007.tfc.util.calendar.CalendarTFC;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Age;
import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Gender;

@ParametersAreNonnullByDefault
public class EntityChickenTFC extends EntityAnimalOviparous implements IAnimalTFC
{
    private static final int DAYS_TO_ADULTHOOD = 124;
    private static final int DAYS_TO_HATCH_EGG = 21;

    private static int getRandomGrowth()
    {
        int lifeTimeDays = Constants.RNG.nextInt(DAYS_TO_ADULTHOOD * 4);
        return (int) (CalendarTFC.PLAYER_TIME.getTotalDays() - lifeTimeDays);
    }
    //Copy from vanilla's EntityChicken, used by renderer to properly handle wing flap
    public float wingRotation;
    public float destPos;
    public float oFlapSpeed;
    public float oFlap;
    public float wingRotDelta = 1.0F;

    public EntityChickenTFC(World worldIn)
    {
        this(worldIn, Gender.fromBool(Constants.RNG.nextBoolean()),
            getRandomGrowth());
    }

    public EntityChickenTFC(World worldIn, Gender gender, int birthDay)
    {
        super(worldIn, gender, birthDay);
        this.func_70105_a(0.9F, 0.9F);
    }

    @Override
    public boolean isValidSpawnConditions(Biome biome, float temperature, float rainfall)
    {
        return temperature > 18 && rainfall > 350;
    }

    @Override
    public void func_70636_d()
    {
        super.func_70636_d();
        if (this.getGender() == Gender.MALE && !this.field_70170_p.field_72995_K && !this.func_70631_g_() && CalendarTFC.CALENDAR_TIME.getHourOfDay() == 6 && CalendarTFC.CALENDAR_TIME.getMinuteOfHour() == 0)
        {
            this.field_70170_p.func_184133_a(null, this.func_180425_c(), TFCSoundEvents.ANIMAL_ROOSTER_CRY, SoundCategory.AMBIENT, 1.0F, 1.0F);
        }
        this.oFlap = this.wingRotation;
        this.oFlapSpeed = this.destPos;
        this.destPos = (float) ((double) this.destPos + (double) (this.field_70122_E ? -1 : 4) * 0.3D);
        this.destPos = MathHelper.func_76131_a(this.destPos, 0.0F, 1.0F);

        if (!this.field_70122_E && this.wingRotDelta < 1.0F)
        {
            this.wingRotDelta = 1.0F;
        }

        this.wingRotDelta = (float) ((double) this.wingRotDelta * 0.9D);

        if (!this.field_70122_E && this.field_70181_x < 0.0D)
        {
            this.field_70181_x *= 0.6D;
        }

        this.wingRotation += this.wingRotDelta * 2.0F;
    }

    @Override
    public float getPercentToAdulthood()
    {
        if (this.getAge() != Age.CHILD) return 1;
        double value = (CalendarTFC.PLAYER_TIME.getTotalDays() - this.getBirthDay()) / (double) DAYS_TO_ADULTHOOD;
        if (value > 1f) value = 1f;
        if (value < 0f) value = 0;
        return (float) value;
    }

    @Override
    public Age getAge()
    {
        return CalendarTFC.PLAYER_TIME.getTotalDays() >= this.getBirthDay() + DAYS_TO_ADULTHOOD ? Age.ADULT : Age.CHILD;
    }

    @Override
    public NonNullList<ItemStack> layEggs()
    {
        NonNullList<ItemStack> eggs = super.layEggs();
        ItemStack egg = new ItemStack(Items.field_151110_aK);
        if (this.isFertilized())
        {
            IEgg cap = egg.getCapability(CapabilityEgg.CAPABILITY, null);
            if (cap != null)
            {
                cap.setFertilized(new EntityChickenTFC(this.field_70170_p), DAYS_TO_HATCH_EGG + CalendarTFC.PLAYER_TIME.getTotalDays());
            }
        }
        this.setFertilized(false);
        eggs.add(egg);
        return eggs;
    }

    @Override
    protected SoundEvent func_184601_bQ(DamageSource damageSourceIn)
    {
        return SoundEvents.field_187666_Z;
    }

    @Override
    protected SoundEvent func_184615_bR()
    {
        return SoundEvents.field_187663_X;
    }

    @Override
    protected void func_184651_r()
    {
        this.field_70714_bg.func_75776_a(0, new EntityAISwimming(this));
        this.field_70714_bg.func_75776_a(1, new EntityAIPanic(this, 1.25D));
        this.field_70714_bg.func_75776_a(2, new EntityAIMate(this, 1D));
        for (ItemStack is : OreDictionary.getOres("grain"))
        {
            Item item = is.func_77973_b();
            this.field_70714_bg.func_75776_a(3, new EntityAITempt(this, 1.1D, item, false));
        }
        this.field_70714_bg.func_75776_a(4, new EntityAIFindNest(this, 1D));
        this.field_70714_bg.func_75776_a(5, new EntityAIWanderAvoidWater(this, 1.0D));
        this.field_70714_bg.func_75776_a(6, new EntityAIWatchClosest(this, EntityPlayer.class, 6.0F));
        this.field_70714_bg.func_75776_a(7, new EntityAILookIdle(this));
    }

    @Override
    protected void func_110147_ax()
    {
        super.func_110147_ax();
        this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111128_a(4.0D);
        this.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(0.2D);
    }

    @Override
    protected SoundEvent func_184639_G()
    {
        return SoundEvents.field_187660_W;
    }

    @Nullable
    protected ResourceLocation func_184647_J()
    {
        return LootTablesTFC.ANIMALS_CHICKEN;
    }

    @Override
    protected void func_180429_a(BlockPos pos, Block blockIn)
    {
        this.func_184185_a(SoundEvents.field_187538_aa, 0.15F, 1.0F);
    }
}
