/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items.metal;

import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.api.util.IPlaceableItem;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.te.TEIngotPile;
import net.dries007.tfc.util.Helpers;

public class ItemIngot extends ItemMetal implements IPlaceableItem
{
    public ItemIngot(Metal metal, Metal.ItemType type)
    {
        super(metal, type);
    }

    @Override
    public boolean doesSneakBypassUse(ItemStack stack, IBlockAccess world, BlockPos pos, EntityPlayer player)
    {
        return true;
    }

    @Override
    public boolean placeItemInWorld(World world, BlockPos pos, ItemStack stack, EntityPlayer player, EnumFacing facing, Vec3d hitVec)
    {
        if (!player.func_70093_af()) return false;

        ItemIngot item = (ItemIngot) stack.func_77973_b();
        if (world.func_180495_p(pos).func_177230_c() != BlocksTFC.INGOT_PILE)
        {
            if (facing == EnumFacing.UP && world.func_180495_p(pos).isSideSolid(world, pos, EnumFacing.UP))
            {
                BlockPos up = pos.func_177984_a();
                if (world.func_190527_a(BlocksTFC.INGOT_PILE, up, false, EnumFacing.UP, null))
                {
                    if (!world.field_72995_K)
                    {
                        world.func_175656_a(up, BlocksTFC.INGOT_PILE.func_176223_P());
                        TEIngotPile te = Helpers.getTE(world, up, TEIngotPile.class);
                        if (te != null)
                        {
                            te.setMetal(item.metal);
                            te.setCount(1);
                        }
                        world.func_184133_a(null, up, SoundEvents.field_187692_g, SoundCategory.BLOCKS, 0.3F, 1.5F);
                    }
                    return true;
                }
            }
        }
        else
        {
            // Place an ingot pile on top of the existing one
            BlockPos posTop = pos.func_177977_b();
            IBlockState stateTop;
            do
            {
                posTop = posTop.func_177984_a();
                stateTop = world.func_180495_p(posTop);
                if (stateTop.func_177230_c() == BlocksTFC.INGOT_PILE)
                {
                    TEIngotPile te = Helpers.getTE(world, posTop, TEIngotPile.class);
                    if (te != null && te.getCount() < 64 && (te.getMetal() == item.metal) && world.func_72855_b(new AxisAlignedBB(0, 0, 0, 1, (1 + te.getCount()) / 64d, 1).func_186670_a(posTop)))
                    {
                        te.setCount(te.getCount() + 1);
                        world.func_184133_a(null, posTop, SoundEvents.field_187692_g, SoundCategory.BLOCKS, 0.3F, 1.5F);
                        return true;
                    }
                }
                else if (stateTop.func_177230_c().func_176200_f(world, posTop) && world.func_190527_a(BlocksTFC.INGOT_PILE, posTop, false, EnumFacing.UP, null))
                {
                    world.func_175656_a(posTop, BlocksTFC.INGOT_PILE.func_176223_P());
                    TEIngotPile te = Helpers.getTE(world, posTop, TEIngotPile.class);
                    if (te != null)
                    {
                        te.setMetal(item.metal);
                        te.setCount(1);
                    }
                    world.func_184133_a(null, posTop, SoundEvents.field_187692_g, SoundCategory.BLOCKS, 0.3F, 1.5F);
                    return true;
                }
                else
                {
                    return false;
                }

            } while (posTop.func_177956_o() <= 256);
        }
        return false;
    }
}
