/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.recipes;

import javax.annotation.Nonnull;

import com.google.gson.JsonObject;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.crafting.Ingredient;
import net.minecraft.util.JsonUtils;
import net.minecraft.util.NonNullList;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.common.ForgeHooks;
import net.minecraftforge.common.crafting.CraftingHelper;
import net.minecraftforge.common.crafting.IRecipeFactory;
import net.minecraftforge.common.crafting.JsonContext;
import net.minecraftforge.oredict.ShapelessOreRecipe;

public class ShapelessDamageAddItemRecipe extends ShapelessOreRecipe
{
    private ItemStack out;
    private int damage;

    public ShapelessDamageAddItemRecipe(ResourceLocation group, NonNullList<Ingredient> input, @Nonnull ItemStack result, @Nonnull ItemStack output, int damage)
    {
        super(group, input, result);
        this.isSimple = false;
        this.out = output;
        this.damage = damage;
    }

    @Override
    @Nonnull
    public NonNullList<ItemStack> func_179532_b(final InventoryCrafting inventoryCrafting)
    {
        final NonNullList<ItemStack> remainingItems = NonNullList.func_191197_a(inventoryCrafting.func_70302_i_(), ItemStack.field_190927_a);

        for (int i = 0; i < remainingItems.size(); ++i)
        {
            final ItemStack itemstack = inventoryCrafting.func_70301_a(i);

            // If the stack isn't empty and the stack is damageable we can damage it, otherwise delegate to containerItem.
            if (!itemstack.func_190926_b() && itemstack.func_77973_b().func_77645_m())
            {
                remainingItems.set(i, damageStack(itemstack));
            }
            else
            {
                remainingItems.set(i, ForgeHooks.getContainerItem(itemstack));
            }
        }

        EntityPlayer player = ForgeHooks.getCraftingPlayer();
        if (player != null)
        {
            player.func_191521_c(out.func_77946_l());
        }

        return remainingItems;
    }

    @Override
    @Nonnull
    public String func_193358_e()
    {
        return group == null ? "" : group.toString();
    }

    // We need to damage the stack but damageItem will not return an item.
    private ItemStack damageStack(ItemStack stack)
    {
        ItemStack damagedStack = stack.func_77946_l();
        damagedStack.func_77972_a(damage, ForgeHooks.getCraftingPlayer());

        return damagedStack;
    }

    public static class Factory implements IRecipeFactory
    {
        @Override
        public IRecipe parse(final JsonContext context, final JsonObject json)
        {
            final String group = JsonUtils.func_151219_a(json, "group", "");
            final NonNullList<Ingredient> ingredients = RecipeUtils.parseShapeless(context, json);
            final ItemStack result = CraftingHelper.getItemStack(JsonUtils.func_152754_s(json, "result"), context);
            final ItemStack output = CraftingHelper.getItemStack(JsonUtils.func_152754_s(json, "output"), context);
            final int damage = JsonUtils.func_151203_m(json, "damage");

            return new ShapelessDamageAddItemRecipe(group.isEmpty() ? null : new ResourceLocation(group), ingredients, result, output, damage);
        }
    }
}
