/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.nbt.NBTTagCompound;

import net.dries007.tfc.util.agriculture.Crop;

@ParametersAreNonnullByDefault
public class TECropSpreading extends TETickCounter
{
    private int maxGrowthStage; // The max value this crop can grow to
    private int baseAge; // The current age, including all spreading attempts
    private boolean isSeedPlant; // Was the plant the initial one that was planted? (controls whether it should drop a seed or not)

    public TECropSpreading()
    {
        this.maxGrowthStage = 0;
        this.isSeedPlant = true;
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt)
    {
        maxGrowthStage = nbt.func_74762_e("maxGrowthStage");
        baseAge = nbt.func_74762_e("baseAge");
        isSeedPlant = nbt.func_74767_n("isSeedPlant");
        super.func_145839_a(nbt);
    }

    @Nonnull
    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound nbt)
    {
        nbt.func_74768_a("maxGrowthStage", maxGrowthStage);
        nbt.func_74768_a("baseAge", baseAge);
        nbt.func_74757_a("isSeedPlant", isSeedPlant);
        return super.func_189515_b(nbt);
    }

    public void onPlaced()
    {
        // Calculate initial max growth stage
        maxGrowthStage = 3 + field_145850_b.func_180495_p(field_174879_c).func_177229_b(Crop.STAGE_8);
        if (maxGrowthStage >= 8)
        {
            maxGrowthStage = 7;
        }
        // Reset counter
        resetCounter();
    }

    public int getMaxGrowthStage()
    {
        return maxGrowthStage;
    }

    public void setMaxGrowthStage(int maxGrowthStage)
    {
        this.maxGrowthStage = maxGrowthStage;
        if (this.maxGrowthStage > 7)
        {
            this.maxGrowthStage = 7;
        }
        func_70296_d();
    }

    public int getBaseAge()
    {
        return baseAge;
    }

    public void setBaseAge(int baseAge)
    {
        this.baseAge = baseAge;
        func_70296_d();
    }

    public boolean isSeedPlant()
    {
        return isSeedPlant;
    }

    public void setSeedPlant(boolean seedPlant)
    {
        isSeedPlant = seedPlant;
        func_70296_d();
    }
}
