/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.ItemStackHandler;

import net.dries007.tfc.objects.inventory.capability.ISlotCallback;
import net.dries007.tfc.objects.inventory.capability.ItemStackHandlerCallback;

/**
 * This is a helper class for TE's with a simple inventory that will respect automation
 * To provide side based automation, you must expose a IItemHandler wrapper based on the input side
 * Without overriding the getCapability methods, this will not accept items from external automation
 */
@ParametersAreNonnullByDefault
public abstract class TEInventory extends TEBase implements ISlotCallback
{
    protected final ItemStackHandler inventory;

    TEInventory(int inventorySize)
    {
        inventory = new ItemStackHandlerCallback(this, inventorySize);
    }

    TEInventory(ItemStackHandler inventory)
    {
        this.inventory = inventory;
    }

    @Override
    public void setAndUpdateSlots(int slot)
    {
        this.func_70296_d();
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt)
    {
        inventory.deserializeNBT(nbt.func_74775_l("inventory"));
        super.func_145839_a(nbt);
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound nbt)
    {
        nbt.func_74782_a("inventory", inventory.serializeNBT());
        return super.func_189515_b(nbt);
    }

    @Override
    public boolean hasCapability(Capability<?> capability, @Nullable EnumFacing facing)
    {
        return (capability == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY && facing == null) || super.hasCapability(capability, facing);
    }

    @Override
    @SuppressWarnings("unchecked")
    public <T> T getCapability(Capability<T> capability, @Nullable EnumFacing facing)
    {
        if (capability == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY && facing == null)
        {
            return (T) inventory;
        }
        return super.getCapability(capability, facing);
    }

    public void onBreakBlock(World world, BlockPos pos)
    {
        for (int i = 0; i < inventory.getSlots(); i++)
        {
            InventoryHelper.func_180173_a(world, pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), inventory.getStackInSlot(i));
        }
    }

    /**
     * Delegated from {@link net.minecraft.inventory.Container#canInteractWith(EntityPlayer)}
     */


    public boolean canInteractWith(EntityPlayer player)
    {
        return this.field_145850_b.func_175625_s(field_174879_c) == this && player.func_70092_e(field_174879_c.func_177958_n() + 0.5D, field_174879_c.func_177956_o() + 0.5D, field_174879_c.func_177952_p() + 0.5D) <= 64.0D;
    }
}
