/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.inventory.ItemStackHelper;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBow;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ItemTool;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;

import net.dries007.tfc.objects.items.metal.ItemMetalTool;

public class TEToolRack extends TileEntity
{
    /**
     * Modify this method to register items that can be put on a tool rack.
     * TODO: an api for other mods to register their items
     *
     * @param item item to check
     * @return true if this item can be put on a tool rack, false otherwise
     */
    public static boolean isItemEligible(Item item)
    {
        return item instanceof ItemMetalTool
            || item instanceof ItemTool
            || item instanceof ItemBow;
    }

    public static boolean isItemEligible(@Nullable ItemStack item)
    {
        if (item == null || item.func_190926_b()) return false;
        return isItemEligible(item.func_77973_b());
    }

    private final NonNullList<ItemStack> items = NonNullList.func_191197_a(4, ItemStack.field_190927_a);

    public NonNullList<ItemStack> getItems()
    {
        return items;
    }

    public void onBreakBlock()
    {
        items.forEach(i -> InventoryHelper.func_180173_a(field_145850_b, field_174879_c.func_177958_n(), field_174879_c.func_177956_o(), field_174879_c.func_177952_p(), i));
    }

    @Override
    public void func_145839_a(NBTTagCompound compound)
    {
        super.func_145839_a(compound);
        ItemStackHelper.func_191283_b(compound.func_74775_l("items"), items);
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound compound)
    {
        super.func_189515_b(compound);
        compound.func_74782_a("items", ItemStackHelper.func_191282_a(new NBTTagCompound(), items));
        return compound;
    }

    @Nullable
    @Override
    public SPacketUpdateTileEntity func_189518_D_()
    {
        return new SPacketUpdateTileEntity(field_174879_c, 127, func_189517_E_());
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189517_E_()
    {
        return func_189515_b(new NBTTagCompound());
    }

    @Override
    public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity pkt)
    {
        func_145839_a(pkt.func_148857_g());
        updateBlock();
    }

    public void updateBlock()
    {
        IBlockState state = field_145850_b.func_180495_p(field_174879_c);
        field_145850_b.func_184138_a(field_174879_c, state, state, 3); // sync TE
        func_70296_d(); // make sure everything saves to disk
    }

    public boolean onRightClick(EntityPlayer player, EnumHand hand, int slot)
    {
        ItemStack slotItem = items.get(slot);
        ItemStack heldItem = player.func_184586_b(hand);
        //we take
        if (!slotItem.func_190926_b())
        {
            //TODO: pit kilns should be this smart too.
            if (player.func_184586_b(hand).func_190926_b())
                player.func_184611_a(hand, slotItem.func_77979_a(1));
            else
            {
                //check for the other hand
                if (hand == EnumHand.MAIN_HAND)
                    hand = EnumHand.OFF_HAND;
                else
                    hand = EnumHand.MAIN_HAND;
                if (player.func_184586_b(hand).func_190926_b())
                    player.func_184611_a(hand, slotItem.func_77979_a(1));
                else
                    player.func_191521_c(slotItem.func_77979_a(1));
            }
            items.set(slot, ItemStack.field_190927_a);
            updateBlock();
            return true;
        }
        //we put
        if (isItemEligible(heldItem))
        {
            items.set(slot, player.func_184812_l_() ?
                new ItemStack(heldItem.func_77973_b(), 1, heldItem.func_77960_j(), heldItem.func_77978_p())
                : heldItem.func_77979_a(1));
            updateBlock();
            return true;
        }
        return false;
    }
}
