/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.util.json;

import java.lang.reflect.Type;
import java.util.HashSet;
import java.util.Set;

import com.google.gson.*;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.util.JsonUtils;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.common.registry.ForgeRegistries;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.registries.TFCRegistries;
import net.dries007.tfc.api.types.Ore;
import net.dries007.tfc.api.types.Rock;
import net.dries007.tfc.api.types.RockCategory;
import net.dries007.tfc.world.classic.worldgen.vein.VeinType;

public class VeinTypeJson implements JsonDeserializer<VeinType>
{
    @SuppressWarnings("deprecation")
    @Override
    public VeinType deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context) throws JsonParseException
    {
        JsonObject jsonObject = JsonUtils.func_151210_l(json, "vein");

        int minY = JsonUtils.func_151203_m(jsonObject, "minimum_height");
        int maxY = JsonUtils.func_151203_m(jsonObject, "maximum_height");
        if (minY > maxY)
        {
            throw new JsonParseException("minimum height cannot be greater than maximum height!");
        }
        int rarity = JsonUtils.func_151203_m(jsonObject, "rarity");
        int density = JsonUtils.func_151203_m(jsonObject, "density");

        VeinType.Size size = VeinType.Size.valueOf(JsonUtils.func_151200_h(jsonObject, "size").toUpperCase());
        VeinType.Shape shape = VeinType.Shape.valueOf(JsonUtils.func_151200_h(jsonObject, "shape").toUpperCase());

        JsonArray rocks = JsonUtils.func_151214_t(jsonObject, "base_rocks");
        Set<Rock> blocks = new HashSet<>();
        rocks.forEach(e -> {
            ResourceLocation name = new ResourceLocation(e.getAsString());
            Rock rock = TFCRegistries.ROCKS.getValue(name);
            if (rock == null)
            {
                RockCategory category = TFCRegistries.ROCK_CATEGORIES.getValue(name);
                if (category == null)
                {
                    TerraFirmaCraft.getLog().warn("Problem parsing ore entry '{}'. Rock / Rock Category '{}' is not defined. Skipping.", name, e);
                }
                else
                {
                    blocks.addAll(category.getRocks());
                }
            }
            else
            {
                blocks.add(rock);
            }
        });

        ResourceLocation oreName = new ResourceLocation(JsonUtils.func_151200_h(jsonObject, "ore"));
        Ore ore = TFCRegistries.ORES.getValue(oreName);
        if (ore == null)
        {
            Block block = ForgeRegistries.BLOCKS.getValue(oreName);
            if (block != null)
            {
                //todo: remove metadata in 1.13
                int meta = JsonUtils.func_151208_a(jsonObject, "meta", 0);
                IBlockState oreState;
                try
                {
                    oreState = block.func_176203_a(meta);
                }
                catch (RuntimeException e)
                {
                    throw new JsonParseException("Unable to find a matching IBlockState for block " + oreName + " and metadata: " + meta);
                }
                return new VeinType.Special(oreState, size, shape, blocks, rarity, minY, maxY, density);
            }
            else
            {
                throw new JsonParseException("Unrecognized ore '" + oreName + "'");
            }
        }
        return new VeinType(ore, size, shape, blocks, rarity, minY, maxY, density);
    }
}
